<?php

namespace Drupal\opigno_statistics\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\group\Entity\GroupInterface;
use Drupal\opigno_statistics\StatisticsPageTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements the training statistics page.
 */
class TrainingForm extends FormBase {

  use StatisticsPageTrait;

  /** @var \Drupal\Core\Database\Connection $database */
  protected $database;

  /** @var \Drupal\Component\Datetime\Time $time */
  protected $time;

  /** @var \Drupal\Core\Datetime\DateFormatter $date_formatter */
  protected $date_formatter;

  public function __construct(
    Connection $database,
    TimeInterface $time,
    DateFormatterInterface $date_formatter
  ) {
    $this->database = $database;
    $this->time = $time;
    $this->date_formatter = $date_formatter;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('datetime.time'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'opigno_statistics_training_form';
  }

  /**
   * Builds training progress.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime $datetime
   *
   * @return array
   *   Render array.
   */
  protected function buildTrainingsProgress(GroupInterface $group, $datetime) {
    $time_str = $datetime->format(DrupalDateTime::FORMAT);

    $query = $this->database->select('opigno_learning_path_achievements', 'a');
    $query->addExpression('SUM(a.progress) / COUNT(a.progress) / 100', 'progress');
    $query->addExpression('COUNT(a.completed) / COUNT(a.registered)', 'completion');

    $or_group = $query->orConditionGroup();
    $or_group->condition('a.completed', $time_str, '<');
    $or_group->isNull('a.completed');

    $data = $query->condition('a.gid', $group->id())
      ->condition('a.registered', $time_str, '<')
      ->execute()
      ->fetchAssoc();

    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['trainings-progress'],
      ],
      'progress' => $this->buildValueWithIndicator(
        $this->t('Training Progress'),
        $data['progress']
      ),
      'completion' => $this->buildValueWithIndicator(
        $this->t('Training Completion'),
        $data['completion']
      ),
    ];
  }

  /**
   * Builds one block for the user metrics.
   *
   * @param string $label
   * @param string $value
   *
   * @return array
   *   Render array.
   */
  protected function buildUserMetric($label, $value) {
    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['user-metric'],
      ],
      [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#attributes' => [
          'class' => ['user-metric-value'],
        ],
        '#value' => $value,
      ],
      [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#attributes' => [
          'class' => ['user-metric-label'],
        ],
        '#value' => $label,
      ],
    ];
  }

  /**
   * Builds user metrics.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *
   * @return array
   *   Render array.
   */
  protected function buildUserMetrics(GroupInterface $group) {
    $query = $this->database->select('users', 'u');
    $query->innerJoin(
      'group_content_field_data',
      'gc',
      "gc.entity_id = u.uid
AND gc.type IN ('learning_path-group_membership', 'opigno_course-group_membership')
AND gc.gid = :gid",
      [
        ':gid' => $group->id(),
      ]
    );
    $users = $query
      ->condition('u.uid', 0, '<>')
      ->countQuery()
      ->execute()
      ->fetchField();

    $now = $this->time->getRequestTime();
    // Last 7 days.
    $period = 60 * 60 * 24 * 7;

    $query = $this->database->select('users_field_data', 'u');
    $query->innerJoin(
      'group_content_field_data',
      'gc',
      "gc.entity_id = u.uid
AND gc.type IN ('learning_path-group_membership', 'opigno_course-group_membership')
AND gc.gid = :gid",
      [
        ':gid' => $group->id(),
      ]
    );
    $new_users = $query
      ->condition('u.uid', 0, '<>')
      ->condition('u.created', $now - $period, '>')
      ->countQuery()
      ->execute()
      ->fetchField();

    $query = $this->database->select('users_field_data', 'u');
    $query->innerJoin(
      'group_content_field_data',
      'gc',
      "gc.entity_id = u.uid
AND gc.type IN ('learning_path-group_membership', 'opigno_course-group_membership')
AND gc.gid = :gid",
      [
        ':gid' => $group->id(),
      ]
    );
    $active_users = $query
      ->condition('u.uid', 0, '<>')
      ->condition('u.login', $now - $period, '>')
      ->countQuery()
      ->execute()
      ->fetchField();

    $users_block = $this->buildUserMetric($this->t('Users'), $users);
    $new_users_block = $this->buildUserMetric(
      $this->t('New users'),
      $new_users
    );
    $active_users_block = $this->buildUserMetric(
      $this->t('Recently active users'),
      $active_users
    );

    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['user-metrics'],
      ],
      [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#attributes' => [
          'class' => ['user-metrics-title'],
        ],
        '#value' => $this->t('Users metrics'),
      ],
      [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['user-metrics-content'],
        ],
        'users' => $users_block,
        'new_users' => $new_users_block,
        'active_users' => $active_users_block,
      ],
    ];
  }

  /**
   * Builds training content.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *
   * @return array
   *   Render array.
   */
  protected function buildTrainingContent(GroupInterface $group) {
    $query = $this->database
      ->select('opigno_learning_path_achievements', 'a');
    $query->leftJoin(
      'opigno_learning_path_step_achievements',
      's',
      's.gid = a.gid AND s.uid = a.uid'
    );
    $query->leftJoin(
      'opigno_learning_path_step_achievements',
      'sc',
      'sc.id = s.id AND sc.completed IS NOT NULL'
    );
    $query->addExpression('COUNT(sc.uid)', 'completed');
    $query->addExpression('AVG(s.score)', 'score');
    $query->addExpression('AVG(s.time)', 'time');

    $data = $query->fields('s', ['name'])
      ->condition('a.gid', $group->id())
      ->groupBy('s.name')
      ->orderBy('s.name')
      ->execute()
      ->fetchAll();

    $query = $this->database->select('users', 'u');
    $query->innerJoin(
      'group_content_field_data',
      'gc',
      "gc.entity_id = u.uid
AND gc.type IN ('learning_path-group_membership', 'opigno_course-group_membership')
AND gc.gid = :gid",
      [
        ':gid' => $group->id(),
      ]
    );
    $users = $query
      ->condition('u.uid', 0, '<>')
      ->countQuery()
      ->execute()
      ->fetchField();

    $table = [
      '#type' => 'table',
      '#attributes' => [
        'class' => ['statistics-table', 'training-content-list', 'table-striped'],
      ],
      '#header' => [
        $this->t('Step'),
        $this->t('% Completed'),
        $this->t('Avg score'),
        $this->t('Avg time spent'),
      ],
      '#rows' => [],
    ];
    foreach ($data as $row) {
      if (!empty($row->name)) {
        $completed = round(100 * $row->completed / $users);
        $score = round($row->score);
        $time = $row->time > 0
          ? $this->date_formatter->formatInterval($row->time)
          : '-';

        $table['#rows'][] = [
          $row->name,
          $this->t('@completed%', ['@completed' => $completed]),
          $this->t('@score%', ['@score' => $score]),
          $time,
        ];
      }
    }

    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['training-content'],
      ],
      'title' => [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#attributes' => [
          'class' => ['training-content-title'],
        ],
        '#value' => $this->t('Training Content'),
      ],
      'list' => $table,
    ];
  }

  /**
   * Builds users results.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *
   * @return array
   *   Render array.
   */
  protected function buildUsersResults(GroupInterface $group) {
    $query = $this->database->select('users_field_data', 'u');
    $query->innerJoin(
      'group_content_field_data',
      'gc',
      "gc.entity_id = u.uid
AND gc.type IN ('learning_path-group_membership', 'opigno_course-group_membership')
AND gc.gid = :gid",
      [
        ':gid' => $group->id(),
      ]
    );
    $query->leftJoin(
      'opigno_learning_path_achievements',
      'a',
      'a.gid = gc.gid AND a.uid = u.uid'
    );

    $data = $query
      ->fields('u', ['uid', 'name'])
      ->fields('a', ['status', 'score', 'time'])
      ->execute()
      ->fetchAll();

    $table = [
      '#type' => 'table',
      '#attributes' => [
        'class' => ['statistics-table', 'users-results-list', 'table-striped'],
      ],
      '#header' => [
        $this->t('User'),
        $this->t('Score'),
        $this->t('Passed'),
        $this->t('Time spent'),
        $this->t('Details'),
      ],
      '#rows' => [],
    ];
    foreach ($data as $row) {
      $score = isset($row->score) ? $row->score : 0;
      $score = [
        'data' => $this->buildScore($score),
      ];

      $status = isset($row->status) ? $row->status : 'pending';
      $status = [
        'data' => $this->buildStatus($status),
      ];

      $time = $row->time > 0
        ? $this->date_formatter->formatInterval($row->time)
        : '-';

      $details_link = Link::createFromRoute(
        '',
        'opigno_statistics.user',
        [
          'user' => $row->uid,
        ]
      )->toRenderable();
      $details_link['#attributes']['class'][] = 'details';
      $details_link = [
        'data' => $details_link,
      ];

      $table['#rows'][] = [
        $row->name,
        $score,
        $status,
        $time,
        $details_link,
      ];
    }
$i = 1;
    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['users-results'],
      ],
      'title' => [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#attributes' => [
          'class' => ['users-results-title'],
        ],
        '#value' => $this->t('Users results'),
      ],
      'list' => $table,
    ];
  }

  /**
   * @param \Drupal\group\Entity\Group $group
   *
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $group = NULL) {
    $form['#title'] = $this->t('Training statistics - @training', [
      '@training' => $group->label(),
    ]);

    $query = $this->database
      ->select('opigno_learning_path_achievements', 'a');
    $query->addExpression('YEAR(a.registered)', 'year');
    $data = $query->condition('a.gid', $group->id())
      ->groupBy('year')
      ->orderBy('year', 'DESC')
      ->execute()
      ->fetchAll();
    $years = [];
    foreach ($data as $row) {
      $year = $row->year;
      if (!isset($years[$year])) {
        $years[$year] = $year;
      }
    }
    $year_select = [
      '#type' => 'select',
      '#options' => $years,
      '#default_value' => max(array_keys($years)),
      '#ajax' => [
        'event' => 'change',
        'callback' => '::submitFormAjax',
        'wrapper' => 'statistics-trainings-progress',
      ],
    ];
    $year = $form_state->getValue('year', max(array_keys($years)));

    $query = $this->database
      ->select('opigno_learning_path_achievements', 'a');
    $query->addExpression('MONTH(a.registered)', 'month');
    $data = $query->condition('a.gid', $group->id())
      ->groupBy('month')
      ->orderBy('month')
      ->execute()
      ->fetchAll();
    $months = [];
    foreach ($data as $row) {
      $month = $row->month;
      if (!isset($months[$month])) {
        $timestamp = mktime(0, 0, 0, $month, 1);
        $months[$month] = $this->date_formatter
          ->format($timestamp, 'custom', 'F');
      }
    }
    $month_select = [
      '#type' => 'select',
      '#options' => $months,
      '#default_value' => max(array_keys($months)),
      '#ajax' => [
        'event' => 'change',
        'callback' => '::submitFormAjax',
        'wrapper' => 'statistics-trainings-progress',
      ],
    ];
    $month = $form_state->getValue('month', max(array_keys($months)));

    $timestamp = mktime(0, 0, 0, $month, 1, $year);
    $datetime = DrupalDateTime::createFromTimestamp($timestamp);
    $datetime->add(new \DateInterval('P1M'));

    $form['trainings_progress'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'statistics-trainings-progress',
      ],
      'year' => $year_select,
      'month' => $month_select,
      'trainings_progress' => $this->buildTrainingsProgress($group, $datetime),
    ];

    $form[] = [
      '#type' => 'container',
      'users' => $this->buildUserMetrics($group),
      'training_content' => $this->buildTrainingContent($group),
      'user_results' => $this->buildUsersResults($group),
    ];

    $form['#attached']['library'][] = 'opigno_statistics/training';

    return $form;
  }

  public function submitFormAjax(array &$form, FormStateInterface $form_state) {
    return $form['trainings_progress'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

}
